/**
 * Copyright (c) 2022, Salesforce, Inc.,
 * All rights reserved.
 * For full license text, see the LICENSE.txt file
 */

/*
 *  ATTENTION!
 *  THIS IS A GENERATED FILE FROM https://github.com/salesforce-experience-platform-emu/lds-lightning-platform
 *  If you would like to contribute to LDS, please follow the steps outlined in the git repo.
 *  Any changes made to this file in p4 will be automatically overwritten.
 *  *******************************************************************************************
 */
import { setDefaultLuvio, withRegistration, register } from '@salesforce/lds-default-luvio';
import { instrumentLuvio, setLdsNetworkAdapterInstrumentation } from '@salesforce/lds-instrumentation';
import salesforceNetworkAdapter, { instrument } from '@salesforce/lds-network-adapter';
import { InMemoryStore, Environment, Luvio } from '@luvio/engine';
import { makeDurable } from '@luvio/environments';
import { fetchNetworkAdapter } from '@luvio/network-adapter-fetch';
import { IndexedDbDurableStore } from '@luvio/durable-store-idb';

/**
 * A NetworkAdapter HOF that sets location.href to
 * "/login?redirect_uri=<location.origin>/oauth" on 401 responses.
 */
function makeNetworkAdapterRedirectToLoginOn401(baseNetworkAdapter) {
    return (resourceRequest, resourceRequestContext) => {
        return baseNetworkAdapter(resourceRequest, resourceRequestContext).then((response) => {
            if (response.status === 401) {
                const redirectUrl = `${location.origin}/oauth`;
                // eslint-disable-next-line no-console
                console.log(`401 response detected, re-directing to /login page`);
                location.href = `/login?redirect_uri=${encodeURIComponent(redirectUrl)}`;
            }
            return response;
        });
    };
}

/**
 * A NetworkAdapter HOF that works with salesforce Core authentication sid cookie
 */
function makeNetworkAdapterCoreAuthenticated(baseNetworkAdapter) {
    return (resourceRequest, resourceRequestContext) => {
        const { headers } = resourceRequest;
        const headersWithAuth = buildHeaders(headers);
        resourceRequest.headers = headersWithAuth;
        return baseNetworkAdapter(resourceRequest, resourceRequestContext);
    };
}
function buildHeaders(additionalHeaders = {}) {
    const headers = {
        ...additionalHeaders,
    };
    if (headers.Authorization === undefined && document && document.cookie) {
        // TODO [W-123]: need to work with auth team to come up with better way
        // to do this... if LWR@Core followed the pattern that LWR@node auth middleware
        // follows then this wouldn't be needed
        const sidCookie = document.cookie.match(new RegExp('(^| )' + 'sid' + '=([^;]+)'));
        if (sidCookie) {
            const sid = sidCookie[2];
            headers['Authorization'] = `Bearer ${sid}`;
        }
    }
    return headers;
}

// so eslint doesn't complain about nimbus
// eslint-disable-next-line @typescript-eslint/no-unused-vars
/* global __nimbus */
let lazyLuvio;
let lazyEnvironment;
/**
 * This returns the LDS on Browser Runtime singleton object.
 */
function getRuntime() {
    if (lazyLuvio === undefined || lazyEnvironment === undefined) {
        const store = new InMemoryStore();
        const networkAdapter = makeNetworkAdapterCoreAuthenticated(makeNetworkAdapterRedirectToLoginOn401(salesforceNetworkAdapter(
        // TODO [W-123]: move the record field chunking code out of nimbus network
        // adapter package and into the shared lds-network-adapter package
        // makeNetworkAdapterChunkRecordFields(fetchNetworkAdapter)
        fetchNetworkAdapter)));
        const baseEnv = new Environment(store, networkAdapter);
        lazyEnvironment = makeDurable(baseEnv, {
            durableStore: new IndexedDbDurableStore(),
        });
        lazyEnvironment.setDefaultCachePolicy({ type: 'cache-then-network' });
        lazyLuvio = new Luvio(lazyEnvironment, {
            instrument: instrumentLuvio,
        });
    }
    return {
        luvio: lazyLuvio,
        environment: lazyEnvironment,
    };
}
function setupLuvio() {
    const { luvio } = getRuntime();
    setDefaultLuvio({ luvio });
}

// defer instrumentation setup on these modules unless/until other code causes
// them to be loaded
withRegistration('@salesforce/lds-network-adapter', (reg) => setLdsNetworkAdapterInstrumentation(reg));
// This code *should* be in lds-network-adapter, but when combined with the Aura
// component test workaround in lds-default-luvio it creates a circular dependency
// between lds-default-luvio and lds-network-adapter. We do the register on behalf
// of lds-network-adapter here to avoid the cycle.
register({
    id: '@salesforce/lds-network-adapter',
    instrument: instrument,
});
// This loader hook sets the luvio instance for running in the browser.
// This must run before any adapters are invoked, otherwise the adapter will be
// undefined (adapter factories need a luvio instance to build adapters)
function ldsRuntimeLoader(_serviceAPI) {
    setupLuvio();
}

export { ldsRuntimeLoader as default, getRuntime };
// version: 1.332.0-dev13-0fd2008fda
